
#include "datastruct.h"

#include "config.h"
#define dilatation_size config.dilatation_size

float distance(Vertex *vtx1, Vertex *vtx2)
{
	return sqrt((float)sqr(vtx1->x - vtx2->x) + (float)sqr(vtx1->y - vtx2->y));
}

GlobalMap::GlobalMap(char *imageName, float cellSize): cellSize(cellSize)
{
	// read image
	Image img;
	if (!img.readBMP(imageName)) std::cout << "image failed to be read !" << std::endl;
	w = img.w, h = img.h;
	map = new char*[w];
	for(int j = 0; j < w; j++) map[j] = new char[h];
	for(int x = 0; x < w; x++) for(int y = 0; y < h; y++) map[x][y] = 0;
	
	// transfer to map with dilatation
	for(int x = 0; x < w; x++) for(int y = 0; y < h; y++) 
	{
		if (img.data[3*(y*w+x)+0] <= 10 && img.data[3*(y*w+x)+1] <= 10 && img.data[3*(y*w+x)+2] <= 10)
		{ // obstacle
			for(int j = -dilatation_size; j <= dilatation_size; j++) 
			for(int i = -dilatation_size; i <= dilatation_size; i++)
				if (x+j >= 0 && x+j < w && y+i >= 0 && y+i < h)
					if (map[x+j][(h-1)-(y+i)] == 0) map[x+j][(h-1)-(y+i)] = 2;
			map[x][(h-1)-y] = 1;
		}
	}
	
	// graph
	nvertices = 0;
	std::list<Vertex*> vert_list;
	for(int x = 0; x < w; x++) for(int y = 0; y < h; y++) 
	{
		if ((map[x][y] == 0) &&
				(map_(x+1,y+1) != 0 || map_(x+1,y-1) != 0 || map_(x-1,y+1) != 0 || map_(x-1,y-1) != 0) &&
				(map_(x+1,y  ) == 0 && map_(x-1,y  ) == 0 && map_(x  ,y+1) == 0 && map_(x  ,y-1) == 0))
		{
			int sides[4] = {0,0,0,0};
			sides[0] = (map_(x+2,y-1)>0?1:0)+(map_(x+2,y+0)>0?1:0)+(map_(x+2,y+1)>0?1:0);
			sides[1] = (map_(x-2,y-1)>0?1:0)+(map_(x-2,y+0)>0?1:0)+(map_(x-2,y+1)>0?1:0);
			sides[2] = (map_(x-1,y-2)>0?1:0)+(map_(x+0,y-2)>0?1:0)+(map_(x+1,y-2)>0?1:0);
			sides[3] = (map_(x-1,y+2)>0?1:0)+(map_(x+0,y+2)>0?1:0)+(map_(x+1,y+2)>0?1:0);
			int ng1 = 0;
			for(int i = 0; i < 4; i++) if (sides[i] > 1) ng1++;
			
			if (ng1 <= 1)
			{
				Vertex *vertex = new Vertex;
				vertex->x = x; vertex->y = y;
				vert_list.push_back(vertex);
				nvertices++;
			}
		}
	}

	
	// convert the list into array (random access, easier to qsort)
	goal_vtx = new Vertex;
	nvertices++;
	vertices = new Vertex*[nvertices];
	vertices[0] = goal_vtx;
	int i = 1;
	for(std::list<Vertex*>::iterator it = vert_list.begin(); it != vert_list.end(); it++, i++)
		vertices[i] = *it;

	// complete graph (find neighbors)
	for(int i = 1; i < nvertices; i++)
	{
		for(int j = i+1; j < nvertices; j++)
		{
			if (areNeighbors(vertices[i]->x, vertices[i]->y, vertices[j]->x, vertices[j]->y))
			{
				Vertex::NeighborVertex vtx(vertices[j], distance(vertices[i], vertices[j]), true);
				vertices[i]->neighbors.push_back(vtx);
				vtx.ptr = vertices[i];
				vertices[j]->neighbors.push_back(vtx);
			}
		}
	}
}

void GlobalMap::add_goal_graph(int goalx, int goaly)
{
	goal_vtx->x = goalx;
	goal_vtx->y = goaly;
	for(int j = 0; j < nvertices; j++)
	{
		if (vertices[j] == goal_vtx) continue;
		if (areNeighbors(goal_vtx->x, goal_vtx->y, vertices[j]->x, vertices[j]->y))
		{
			Vertex::NeighborVertex vtx(vertices[j], distance(goal_vtx, vertices[j]), true);
			goal_vtx->neighbors.push_back(vtx);
			vtx.ptr = goal_vtx;
			vertices[j]->neighbors.push_back(vtx);
		}
	}
}

GlobalMap::~GlobalMap()
{
	for(int i = 0; i < w; i++) delete[] map[i];
	delete[] map;
	delete[] vertices;
}



bool GlobalMap::areNeighbors(int x1, int y1, int x2, int y2)
{
count_are_neighbors++;
	double stepx, stepy; int steps;
	if (abs(x2-x1) > abs(y2-y1))
		{ stepx = sign(x2-x1); stepy = (y2-y1)/(float)abs(x2-x1); steps = (int)abs(x2-x1); } else
		{ stepy = sign(y2-y1); stepx = (x2-x1)/(float)abs(y2-y1); steps = (int)abs(y2-y1); }
				
	double x = x1+1*stepx, y = y1+1*stepy;
	for(int i = 1; i < steps; i++, x += stepx, y += stepy)
		if (map[(int)x][(int)y] > 0) return false;

	return true;
}





