#ifndef _DATASTRUCT_H_
#define _DATASTRUCT_H_

#include "lib.h"
#include <list>
#include <queue>


struct Vertex
{
	int x, y; // position
	struct NeighborVertex 
	{ 
		Vertex *ptr; 
		real distance; 
		bool tested_neighbor; // tested if is neighbor of its parent
		bool removed;
		NeighborVertex(Vertex *ptr, real distance, bool tested_neighbor): 
			ptr(ptr), distance(distance), tested_neighbor(tested_neighbor), removed(false) {};
		bool operator<(NeighborVertex &nv) { return ptr->overall_cost < nv.ptr->overall_cost; }
	};
	std::list<NeighborVertex> neighbors;
	// for A*
	float cost; 
	float overall_cost;
	Vertex *parent;
	bool is_in_open;
	bool is_in_closed;
};

typedef std::list<Vertex::NeighborVertex> NeighborList;

float distance(Vertex *vtx1, Vertex *vtx2);


class GlobalMap
{
 public:
	// data
	int w, h; /// width (pix),  height (pix)
	char **map; /// size w*h, map[x][y], 0 = free, 1 = obstacle
	float cellSize;
	int nvertices;
//	std::pair<std::pair<int,int>, std::list<std::pair<int,real> > **vertices; // (1)
	Vertex **vertices; // (2)
	Vertex *goal_vtx;
	// constructors
	GlobalMap(char *imageName, float cellSize);
	~GlobalMap();
	// methods
	char map_(int x, int y) { if ((x >= 0) && (x < w) && (y >= 0) && (y < h)) return map[x][y]; else return 1; }
	bool areNeighbors(int x1, int y1, int x2, int y2);
int count_are_neighbors;
int count_vtx_tested;
	void add_goal_graph(int goalx, int goaly);
};

struct RobotState
{
	float x, y; /// position
	float theta;
	float cos_theta, sin_theta;
	float speed, rot_speed;
	Date timestamp;
};


#endif
