
#include "io.h"
#include "hclient.h"
#include "config.h"
#include "cyril-timer.h"

/*
Robot::Robot(float w, float h, int nsensors): w(w), h(h), nsensors(nsensors)
{
	readings = new SensorReading[nsensors];
//	for(int i = 0; i < nsensors; i++) sensors[i].id = i;
	sonar_outrange = (float)SONAR_READING_OUTSIDE_RANGE;
}
*/
void Robot::init(char *hserver_name)
{
	initState = state;
	initState.cos_theta = cos(initState.theta*2*M_PI/360);
	initState.sin_theta = sin(initState.theta*2*M_PI/360);
	
	for(int i = 0; i < nsensors; i++)
	{
		readings[i].initialized = false;
	}
	
	int res;
	sonar_outrange = (float)SONAR_READING_OUTSIDE_RANGE*100;	
	res = hclient_open_robot(hserver_name, false); if (res != 0) ERROR("HClient error: %d", res);
	res = hclient_init_robot(state.x/100, state.y/100, state.theta); if (res != 0) ERROR("HClient error: %d", res);
	res = hclient_setxyt(state.x/100, state.y/100, state.theta); if (res != 0) ERROR("HClient error: %d", res);
	res = hclient_range_start(); if (res != 0) ERROR("HClient error: %d", res);
}

void Robot::close()
{
	int res;
	res = hclient_stop(); if (res != 0) ERROR("HClient error: %d", res);
	res = hclient_range_stop(); if (res != 0) ERROR("HClient error: %d", res); // stop sonar
	res = hclient_close_robot(); if (res != 0) ERROR("HClient error: %d", res);
}

void Robot::updateReadings()
{
	int res;
	int num_sensors;
	sensor_t *sensors;
	res = hclient_get_sensor_readings(0xFFFF, &num_sensors, &sensors); if (res != 0) ERROR("HClient error: %d", res);

	refreshState();		
	
	int sensor;
	for(sensor = 0; sensor < num_sensors; sensor++)
		if (sensors[sensor].type == SENSOR_SONAR)
		{
			if (sensors[sensor].num_readings != nsensors) std::cout << "not same number of sensors for sensor "
				<< sensor << " (" << sensors[sensor].num_readings << " instead of " << nsensors << ")" << std::endl;
			break;
		}
	
	//
	int num_readings = sensors[sensor].num_readings;
	for(int i = 0; i < num_readings; i++)
	{
		float *pos = (float*)sensors[sensor].extra;
		if((readings[i].value_ != sensors[sensor].reading[i] /*|| 
			  (readings[i].value == sonar_outrange && readings[mod(i-1,num_readings)].updated)*/) || 
			((sensors[sensor].len_extra == num_readings * (int)sizeof(float) * 3) && 
			 (readings[i].x_ != pos[i*3+0] || readings[i].y_ != pos[i*3+1] || 
			  readings[i].theta_ != pos[i*3+2])
			))
		{
			readings[i].angle = sensors[sensor].angle[i];
			readings[i].value_ = sensors[sensor].reading[i];
			readings[i].x_ = pos[i*3+0];
			readings[i].y_ = pos[i*3+1];
			readings[i].theta_ = pos[i*3+2];

			readings[i].value = sensors[sensor].reading[i]*100;
			/* 
			It seems that the dead-reckoning used to tag sonar measures isn't affected by setxyt or init_robot function, 
			so the init position is always 0,0,0. Then we have to do a transformation to get back in the same coordinate system than the robot
			*/
			readings[i].x = (pos[i*3+0]*initState.cos_theta - pos[i*3+1]*initState.sin_theta)*100+initState.x;
			readings[i].y = (pos[i*3+0]*initState.sin_theta + pos[i*3+1]*initState.cos_theta)*100+initState.y;
			readings[i].theta = mod(pos[i*3+2] + initState.theta, 360);
			/*
			use this if you don't want to use position-stamps on measures, but be sure to make a refreshState() before
			of this function
			*/
//			readings[i].x = state.x/100;
//			readings[i].y = state.y/100;
//			readings[i].theta = state.theta;

			readings[i].updated = readings[i].initialized;
			readings[i].initialized = true;
			
DEBUG("new readings: [(%f,%f,%f)(%f,%f) original: (%f,%f,%f)]", 
	readings[i].x, readings[i].y, readings[i].theta, readings[i].angle, readings[i].value,
	pos[i*3+0], pos[i*3+1], pos[i*3+2]);
			
		} else 
			readings[i].updated = false;
	}
	
}



void Robot::refreshState()
{
	double x, y, theta;
	int res;
	res = hclient_getxyt(&x, &y, &theta); if (res != 0) ERROR("HClient error: %d", res);
	Date newtimestamp = get_c_us();
	state.speed = sqrt(sqr(state.x-x*100)+(state.y-y*100))/(newtimestamp-state.timestamp);
	state.rot_speed = (theta-state.theta)/(newtimestamp-state.timestamp);

	state.timestamp = newtimestamp;
	state.x = x*100;
	state.y = y*100;
	state.theta = theta;
}


// absolute theta
void Robot::setDirSpeed(float theta, float speed)
{
	refreshState();	// we need to have the latest value for angle control	

	// compute angle to turn to be aligned with theta
	float angle_left = mod(theta-state.theta, 360);
	float angle_right = mod(state.theta-theta, 360);
	float new_theta = (angle_left < angle_right ? angle_left : -angle_right);
	DEBUG("theta %f, robot.theta %f, angle_left %f, angle_right %f, new_theta %f", 
		theta, state.theta, angle_left, angle_right, new_theta);

	// if control  by speed, do sthg like that
/*		
	float time_rot = 1;//(abs(new_theta) < 20 ? 0.5 : 1);
	float rot_speed = new_theta / time_rot;
	rot_speed = min(state.rot_speed, config.max_rot_speed);
*/

//	speed *= (1 - state.rot_speed/config.max_rot_speed);
//	max(v - config.min_speed, (float)0.)*(1 - rs.rot_speed/config.max_rot_speed) + config.min_speed;
	
	int res;
//	res = hclient_steer(state.rot_speed); if (res != 0) ERROR("HClient error: %d", res);
	res = hclient_steer(new_theta*0.9); if (res != 0) ERROR("HClient error: %d", res);
	res = hclient_drive(speed/100); if (res != 0) ERROR("HClient error: %d", res);
//	res = hclient_steer_toward_and_drive(speed/100, new_theta); if (res != 0) ERROR("HClient error: %d", res);
}


void Robot::stop()
{
	int res;
	res = hclient_stop(); if (res != 0) ERROR("HClient error: %d", res);
}

