/********************************************
Personal library of Cyril Roussillon
http://crteknologies.free.fr
cyril42e@gmail.com
2006-2007
*********************************************/

#include "cyril-std.h"

#include <sys/sysinfo.h>


char* strclone(const char *str)
{
	if (str == NULL) return NULL;
	return strdup(str);
}

char* getfname(const char *str)
{
	char *res = strrchr(str, '/');
	if (res == NULL) return strclone(str);
	else return strclone(res+1);
}

char* getfpath(const char *str)
{
	char *end = strrchr(str, '/');
	if (end == NULL || end == str) return NULL;
	char *res = new_(char, end-str+1);
	strncpy(res, str, end-str);
	res[end-str] = 0;
	return res;
}

char* pathcat(const char *path, const char *file)
{ // concat the path name (without file name) of 'path' with 'file'
	int nfile = file ? strlen(file) : 0;
	char *last = strrchr(path, '/');
	int npath = (last == NULL) ? 0 : last-path+1; // last occurence of '/' in path
	char *res = new_(char, npath+nfile+1); res[0] = 0;
	strncat(res, path, npath); res[npath] = 0;
	strncat(res, file, nfile+1);
	return res;
}

void exec(const char *command, const char *params)
{
	char fullcommand[200];
	sprintf(fullcommand, "%s %s", command, params);
	system(fullcommand);
}

void mkdir(const char *dir)
{
	char tmp_dir[256];
	tmp_dir[0] = 0;
	int len = strlen(dir);
	if (len <= 253) sprintf(tmp_dir, "\"%s\"", dir); else { strncpy(tmp_dir, dir, 255); tmp_dir[255] = 0; }
	exec("mkdir -p", tmp_dir);
}

void rmfiles(const char *files)
{
	exec("rm -rf", files);
}


FILE* fcreate(const char *filename, const char *mode)
{
	FILE *file = fopen(filename, mode);
	if (file == NULL)
	{
		char *path = getfpath(filename);
		mkdir(path);
		delete_(path);
		file = fopen(filename, mode);
	}
	return file;
}

bool fexists(const char *filename)
{
	if (!filename) return false;
	if (filename[strlen(filename)-1] == '/') return false;
	FILE *file = fopen(filename, "rt");
	if (file) fclose(file);
	return file;
}

long strtoi(char *s)
{
	while (*s == ' ' || *s == '\t' || *s == '\n' || *s == '\r') s++;
	if (s[0] == '0' && (s[1]|32) == 'x') return strtol(s+2, NULL, 16);
	if ((s[0]|32) == 'b') return strtol(s+1, NULL, 2);
	if (s[0] == '0') return strtol(s+1, NULL, 8);
	return strtol(s, NULL, 10);
}



/**
renvoie le nombre optimal de threads  faire tourner selon le critre choice :
  >= 0 : nombre impos (1 = 1 thread de calcul soit 2 avec le main, 0 = aucun thread de calcul (fait dans celui du main) donc compltement monothread
  -1 : automatique (nombre de cpu configurs dans l'os)
  -2 : automatique (nombre de cpu disponibles)
  -3 : automatique(nombre de cpu sans charge tenant compte du loadavg)
*/
int get_nthreads(int choice)
{
	if (choice >= 0) return choice;
	int res;
	switch (choice)
	{
		case -1: return ((res = get_nprocs_conf()) > 0 ? res : 1);
		case -2: return ((res = get_nprocs()) > 0 ? res : 1);
		case -3: {
			double loadavg[3];
			if (getloadavg (loadavg, 3) != -1)
			{
				res = get_nprocs();
				if (res <= 0) res = 1;
				res = res - (int)(min3(loadavg[0], loadavg[1], loadavg[2])); // amliorer cette formule, peut etre en donnant plus de choix  l'utilisateur
				if (res <= 0) res = 1;
				return res;
			} else return ((res = get_nprocs()) > 0 ? res : 1);
		}
		default: return ((res = get_nprocs()) > 0 ? res : 1);
	}
}



