/********************************************
Personal library of Cyril Roussillon
http://crteknologies.free.fr
cyril42e@gmail.com
2006-2007
*********************************************/

#include "cyril-struct.h"

//////////////////////////////////////////////////////////////////
// queue
//////////////////////////////////////////////////////////////////
// void __() {}


void allocStack(Stack *s, int size)
{
	s->size = size;
	s->elements = new_(void*, s->size);
	s->head = 0;
}

void freeStack(Stack *s)
{
	s->size = -1;
	delete_(s->elements);
	s->head = 0;
}


bool isStackEmpty(Stack *s)
{
	return (s->head == 0);
	//return (lengthStack(s) == 0);
}
bool isStackFull(Stack *s)
{
	return (s->head == s->size);
	//return (lengthStack(s) == s->size);
}
int lengthStack(Stack *s)
{
	return (s->head);
}

void* popStack(Stack *s)
{
	if (isStackEmpty(s)) return NULL;
	return s->elements[--s->head];
}

void pushStack(Stack *s, void *elt)
{
	if (isStackFull(s))
	{
		s->size *= 2;
		s->elements = renew_(s->elements, void*, s->size);
	}
	s->elements[s->head++] = elt;
}

void flushStack(Stack *s)
{
	s->head = 0;
}


void allocQueue(Queue *q, int size)
{
	q->size = size;
	q->elements = new_(void*, q->size+1);
	q->first = q->last = 0;
}

void reallocQueue(Queue *q, int size)
{
	int oldsize = q->size;
	void* *oldelements = q->elements;
	q->size = size;
	q->elements = new_(void*, q->size+1);
	int step1 = min(q->last, oldsize+1);
	for(int i = q->first, j=0; i < step1; i++, j++) q->elements[j] = oldelements[i];
	int step2 = (q->last >= q->first ? 0 : q->last);
	for(int i = 0, j=step1-q->first; i < step2; i++, j++) q->elements[j] = oldelements[i];
	q->first = 0;
	q->last = oldsize;
	delete_(oldelements);
}


void freeQueue(Queue *q)
{
	q->size = -1;
	delete_(q->elements);
	q->first = q->last = 0;
}


bool isQueueEmpty(Queue *q)
{ // last == first
	return (q->first == q->last);
	//return (lengthQueue(q) == 0);
}
bool isQueueFull(Queue *q)
{ // last == first-1
	return (((q->last+1)%(q->size+1)) == q->first);
	//return (lengthQueue(q) == q->size);
}
int lengthQueue(Queue *q)
{
	return ((q->last >= q->first) ? (q->last-q->first) : (q->size+1+(q->last-q->first)));
}

//void* takeFrontQueue(Queue *q)
void* takeQueue(Queue *q)
{
	if (isQueueEmpty(q)) return NULL;
	void* res = q->elements[q->first];
//	q->first++; if (q->first > q->size) q->first = 0;
	q->first = (q->first+1)%(q->size+1);
	return res;
}

//void* takeBackQueue(Queue *q)
void* popQueue(Queue *q)
{
	if (isQueueEmpty(q)) return NULL;
	q->last = (q->last-1 + q->size+1)%(q->size+1);
	return q->elements[q->last];
}


//void pushBackQueue(Queue *q, void *elt)
void pushQueue(Queue *q, void *elt)
{
	if (isQueueFull(q)) reallocQueue(q, 2*q->size);
	q->elements[q->last] = elt;
	q->last = (q->last+1)%(q->size+1);
}

//void pushFrontQueue(Queue *q, void *elt)
void insertQueue(Queue *q, void *elt)
{
	if (isQueueFull(q)) reallocQueue(q, 2*q->size);
	q->first = (q->first-1 + q->size+1)%(q->size+1);
	q->elements[q->first] = elt;
}


void flushQueue(Queue *q)
{
	q->first = q->last = 0;
}


//////////////////////////////////////////////////////////////////
// string parsing
//////////////////////////////////////////////////////////////////
// void __() {}



int parserGetCount(char *s, char sep)
{
	int count = 1;
	for(char *c = s; *c != 0; c++)
		if (*c == sep) count++;
	return count;
}

/**
	@arg type the type of *data (determines elements size and conversion)
*/
int parserExtract(char *s, char sep, void *data, int count, NativeType type)
{
	bool cut;
	char *begin = s, *end;
	for(int i = 0; i < count; i++)
	{
		for(end = s; *end != 0 && *end != sep; end++);
		if (*end == 0 && i < count-1) return i+1;
		if (*end == sep) { *end = 0; cut = true; } else cut = false;
		switch (type)
		{
			case ntChar:  case ntUChar:  ((char*)         data)[i] = atoi(begin); break;
			case ntBool:                 ((bool*)         data)[i] = atoi(begin); break;
			case ntShort: case ntUShort: ((short*)        data)[i] = atoi(begin); break;
			case ntInt:   case ntUInt:   ((int*)          data)[i] = atoi(begin); break;
			case ntLInt:  case ntULInt:  ((long int*)     data)[i] = atoi(begin); break;
			case ntLLInt: case ntULLInt: ((long long int*)data)[i] = atoi(begin); break;
			case ntFloat:                ((float*)        data)[i] = atof(begin); break;
			case ntDouble:               ((double*)       data)[i] = atof(begin); break;
			case ntLDouble:              ((long double*)  data)[i] = atof(begin); break;
			case ntPtr: default:         ((char**)        data)[i] = strclone(begin);
		}
		if (cut) *end = sep; // restore
		begin = end+1;
	}
	return count;
}



